@@ -1,6 +1,4 @@
 /*
- * $Id: engine.c 7194 2013-07-23 10:46:46Z matthijs $
- *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -758,7 +756,7 @@ dnsconfig_zone(engine_type* engine, zone_type* zone)
     } else if (zone->xfrd) {
         netio_remove_handler(engine->xfrhandler->netio,
             &zone->xfrd->handler);
-        xfrd_cleanup(zone->xfrd);
+        xfrd_cleanup(zone->xfrd, 0);
         zone->xfrd = NULL;
     }
     if (zone->adoutbound->type == ADAPTER_DNS) {
