@@ -1,6 +1,4 @@
 /*
- * $Id: tools.c 7359 2013-10-11 11:55:08Z matthijs $
- *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -136,8 +134,13 @@ tools_input(zone_type* zone)
     start = time(NULL);
     status = adapter_read((void*)zone);
     if (status != ODS_STATUS_OK && status != ODS_STATUS_UNCHANGED) {
-        ods_log_error("[%s] unable to read zone %s: adapter failed (%s)",
-            tools_str, zone->name, ods_status2str(status));
+        if (status == ODS_STATUS_XFRINCOMPLETE) {
+            ods_log_info("[%s] read zone %s: xfr in progress",
+                tools_str, zone->name);
+        } else {
+            ods_log_error("[%s] unable to read zone %s: adapter failed (%s)",
+                tools_str, zone->name, ods_status2str(status));
+        }
         zone_rollback_dnskeys(zone);
         zone_rollback_nsec3param(zone);
         namedb_rollback(zone->db, 0);
@@ -257,19 +260,20 @@ tools_output(zone_type* zone, engine_type* engine)
                 break;
         }
     }
-    if (engine->dnshandler) {
-        dnshandler_fwd_notify(engine->dnshandler, (uint8_t*) ODS_SE_NOTIFY_CMD,
-            strlen(ODS_SE_NOTIFY_CMD));
-    }
     /* log stats */
     if (zone->stats) {
         lock_basic_lock(&zone->stats->stats_lock);
         zone->stats->end_time = time(NULL);
-        ods_log_debug("[%s] log stats for zone %s", tools_str,
-            zone->name?zone->name:"(null)");
-        stats_log(zone->stats, zone->name, zone->signconf->nsec_type);
+        ods_log_debug("[%s] log stats for zone %s serial %u", tools_str,
+            zone->name?zone->name:"(null)", (unsigned) zone->db->outserial);
+        stats_log(zone->stats, zone->name, zone->db->outserial,
+            zone->signconf->nsec_type);
         stats_clear(zone->stats);
         lock_basic_unlock(&zone->stats->stats_lock);
     }
+    if (engine->dnshandler) {
+        dnshandler_fwd_notify(engine->dnshandler, (uint8_t*) ODS_SE_NOTIFY_CMD,
+            strlen(ODS_SE_NOTIFY_CMD));
+    }
     return status;
 }
